#========================================================================
# NAM: TkIMhtml
# ROL: Collection of functions processing HTML files
#
#                            -oOo-
#
#      This source can be distributed without any permission.
#      If you use it, tell me about it !
#
#      Ce source est distribuable sans autorisation.
#      S'il vous a servi, dites le moi !
#
# AUT: AnkiF (ankif@mailoman.com)
#
# $Id: TkIMhtml.tcl,v 2.3 2000/05/14 02:31:33 fran Exp fran $
#
#========================================================================


#========================================================================
# FCT: TkIMhtml:readTag
# ROL: Reads a whole HTML tag, possibly split across several lines.
#      IMPORTANT: the beginning of the tag MUST be in the current
#      buffered line (curligne).
#
# ARG: fd       (I): File handle
#      curligne (I): Content of the current line
#      nomtag   (I): Tag to look for
#
# RET: the whole tag.
#
#========================================================================
proc TkIMhtml:readTag { fd curligne nomtag } {
upvar $curligne maligne

   set fulltag "<$nomtag "
   set count 0
   set total ""

   # Get the whole tag, on several lines if necessary

   set indtag [string first $fulltag [string toupper $maligne]]
   if { $indtag >= 0 } {
      set total [string range $maligne [expr $indtag+[string length $fulltag]] end]
      set indice [string first ">" $total]
      if { $indice >= 0} {
         set maligne [string range $total [expr $indice+1] end]
         set total [string range $total 0 $indice]
        } \
      else {
         while {($indice < 0) && ($count >= 0)} {
            set count [gets $fd maligne]
            set indice [string first ">" $maligne]
            if {$indice <0} { 
                set total $total$maligne 
            } \
            else { 
               set total $total[string range $maligne 0 $indice]
               set maligne [string range $maligne [expr $indice+1] end]
            }
         }
      }
   }


   return $total
}



#========================================================================
# FCT: TkIMhtml:parseTag
#
# ROL: Parses a whole HTML tag, possibly split across several lines.
#      IMPORTANT: the beginning of the tag MUST be in the current
#      buffered line (curligne).
#
# ARG: fd       (I): File handle
#      curligne (M): Content of the current line in input, end of the
#                    line (tag removed) on output
#      nomtag   (I): Tag to look for
#      contenu  (O): Array of subtags and their possible content
#                    if curligne = <BODY TEXTCOLOR=blue LINK=red NOBORDER>
#                    contenu(TEXTCOLOR)=blue
#                    contenu(LINK)=red
#                    contenu(NOBORDER)=""
#
# RET: none.
#
#========================================================================
proc TkIMhtml:parseTag { fd curligne nomtag contenu} {
upvar $curligne maligne
upvar $contenu tabl

   catch { unset tabl }


   set total [ TkIMhtml:readTag $fd maligne $nomtag ]

   # total contains the whole tag, let's parse it

   set liste [split $total]
   foreach elem $liste {

     set elem [string trim $elem "<>"]

     set index_egal [string first "=" $elem]
     if { $index_egal > 0 } {
        set tag [string range $elem 0 [expr $index_egal-1]]
        set val [string range $elem [expr $index_egal+1] end]
     } else {
        set tag $elem
        set val ""
     }
     set tabl($tag) $val
     lappend tabl("tags") $tag
   }
}




#========================================================================
# FCT: TkIMhtml:rebuildTag
#
# ROL: .
#
# ARG: fd       (I): File handle
#      nomtag   (I): Tag to write
#      contenu  (O): Array of subtags and their possible content
#
#      if nomtag = BODY and contenu(TEXTCOLOR)=blue
#                           contenu(LINK)=red
#                           contenu(NOBORDER)=""
#      then create : <BODY TEXTCOLOR=blue LINK=red NOBORDER>
#
# RET: none.
#
#========================================================================
proc TkIMhtml:rebuildTag { fd nomtag contenu} {
upvar $contenu tabl

   puts -nonewline $fd "<$nomtag"

   # I store the tags in a list so as to rewrite
   # them in the correct order ("array names" gives
   # them in an "arbitrary" order)

   for {set i 0} {$i < [llength $tabl("tags")]} {incr i} {
      set elem [lindex $tabl("tags") $i]
      puts -nonewline $fd " $elem=$tabl($elem)"
   }
   
   puts -nonewline $fd ">"

}




#========================================================================
# FCT: TkIMhtml:ReadImgMap
# ROL: Scans an HTML file for an imagemap definition. The first one to
#      be found is read in. The file *must* exist : no error handling.
#
# ARG: fichier  (I): File name
#      nommap   (O): Map name
#      imgmap   (O): Image map : array (0..n) of lists. Each list contains
#                    5 elements : shape, coordinates, URL, ALT, TARGET
#
# RET: "ok" if an imagemap could be found in the file
#
#========================================================================
proc TkIMhtml:readImgMap { fichier nommap imgmap } {
upvar $nommap mapname

   set fd [open $fichier "r"]
   set mapfound ""
   set i 0

   while {[gets $fd ligne] >= 0} {
      set upligne [string toupper $ligne] ;		# conversion into uppercase

      if {$mapfound != "ok"} {
         # Imagemap not found yet : look for it

         # Look for an imagemap tag
         set inddeb [string first "<MAP " $upligne]
         if { $inddeb >= 0 } {
            # Imagemap's name extraction
            set total [TkIMhtml:readTag $fd ligne "MAP"]
            set indice [string first "NAME=" $total]
            if { $indice >= 0 } {
               scan [string range $total [expr $indice+5] end] {%*[\"]%[^\"]} mapname
               set mapfound "ok"
            }
         }
      } \
      else {
         # Imagemap tag already found : read shapes definition


         set upligne [string toupper $ligne]
         set inddeb [string first "<AREA " $upligne]
         if { $inddeb >= 0 } {
             set total [TkIMhtml:readTag $fd ligne "AREA"]
             set upligne [string toupper $total]

             # -- Type of shape --
             set indice [string first "SHAPE" $upligne]
             set forme "rect"
             if { $indice >= 0 } {
                scan [string range $total [expr $indice+5] end] {%*[\"=]%[^\"]} forme
             }

             # -- Points coordinates (space- or comma separated list of numbers) --
             set indice [string first "COORDS" $upligne]
             set htmlcoords ""
             if { $indice >= 0 } {
                scan [string range $total [expr $indice+6] end] {%*[\"=]%[^\"]} htmlcoords
                set coords [split $htmlcoords {, }]
             }

             # -- Associated URL --
             set indice [string first "HREF" $upligne]
             if { $indice >= 0 } {
                set url ""
                scan [string range $total [expr $indice+4] end] {%*[\"=]%[^\"]} url
             }

             # -- Alternate text (ALT) --
             set indice [string first "ALT" $upligne]
             set alt ""
             if { $indice >= 0 } {
                scan [string range $total [expr $indice+3] end] {%*[\"=]%[^\"]} alt
               }

             # -- Target frame --
             set indice [string first "TARGET" $upligne]
             set target ""
             if { $indice >= 0 } {
                scan [string range $total [expr $indice+6] end] {%*[\"=]%[^\"]} target
             }

             # -- Store the area description in the output
	     # -- array of lists

             upvar ${imgmap}($i) dummy
             set dummy [list $forme $coords $url $alt $target]
             incr i
          }
      }
  }

close $fd
return $mapfound
}



#========================================================================
# FCT: TkIMhtml:findImgFile
# ROL: Scans an HTML file and look for a tag <IMG SRC="ficimg" USEMAP="#nommap">
#
# ARG: fichier  (I): HTML file to scan
#      nommap   (I): Map name
#      ficimg   (O): Image file name
# RET: none
#
#========================================================================
proc TkIMhtml:findImgFile { fichier nommap ficimg } {
upvar $ficimg myFile

   set fd [open $fichier "r"]
   set myFile ""

   while {[gets $fd ligne] >= 0} {
      set upligne [string toupper $ligne] ;		# conversion to uppercase

      # Look for an imagemap tag
      set inddeb [string first "<IMG " $upligne]
      if { $inddeb >= 0 } {

         # Read all the tag 
         set total [TkIMhtml:readTag $fd ligne "IMG"]


         # Look for an imagemap
         set indice [string first "USEMAP=\"#$nommap" $total]

         if { $indice >= 0 } {
            set indice [string first "SRC=" $total]
            if { $indice >= 0 } {
               scan [string range $total [expr $indice+4] end] {%*[\"=]%[^\"]} myFile
            }              
            break
         }
      }

   }

close $fd
}



#========================================================================
# FCT: TkIMhtml:InsertImgMapDef
# ROL: Inserts an imagemap definition in an open HTML file
#
# ARG: fd       (I): File descriptor
#      nommap   (I): Map name
#      im       (I): Image map : array (0..n) of lists. Each list contains
#                    5 elements : shape, coordinates, URL, ALT, TARGET
# RET: none 
#
#========================================================================
proc TkIMhtml:InsertImgMapDef { fd nommap im } {
upvar $im imgmap

    puts $fd "<MAP NAME=\"$nommap\">"


    # Loop on the areas (array of lists)

    foreach { indice } [array names imgmap] {

       puts -nonewline $fd "<AREA SHAPE=\""
       set zone [lindex $imgmap($indice) 1]

       switch [lindex $imgmap($indice) 0] {
          rectangle { 
             puts -nonewline $fd "rect" 
          } 
          oval { 
             puts -nonewline $fd "circle"
             set xcentre [expr ([lindex $zone 0]+[lindex $zone 2])/2]
             set ycentre [expr ([lindex $zone 1]+[lindex $zone 3])/2]
             set rayon [expr ([lindex $zone 2]-[lindex $zone 0])/2]
             set zone [list $xcentre $ycentre $rayon]
          } 
          default {
             puts -nonewline $fd "poly"
          }
         }

       set url [lindex $imgmap($indice) 2]
       set alt [lindex $imgmap($indice) 3]
       set target [lindex $imgmap($indice) 4]

       #  Special treatment for the coords : the canvas returns
       #  a list of floats and we prefer a comma-separated
       #  series of integers

       set coords ""
       foreach coord $zone {
          append coords "[expr round($coord)],"
       }

       puts -nonewline $fd "\" COORDS=\"[string trim $coords ,]\" "
       puts -nonewline $fd " HREF=\"$url\" ALT=\"$alt\""
       if { $target != "" } {
          puts -nonewline $fd " TARGET=\"$target\""
       }
       puts $fd ">"
    }


    puts $fd "</MAP>"
}



#========================================================================
# FCT: TkIMhtml:InsertImgMapRef
# ROL: Inserts an imagemap reference in an open HTML file
#
# ARG: fd       (I): File descriptor
#      nommap   (I): Map name
#      ficimg   (I): Image file name
#
# RET: none 
#
#========================================================================
proc TkIMhtml:InsertImgMapRef { fd nommap ficimg } {

    puts $fd "<IMG SRC=\"$ficimg\" USEMAP=\"#$nommap\">"
}


#========================================================================
# FCT: TkIMhtml:UpdateImgMap
# ROL: Updates the current imagemap definition in a file. The output is
#      in HTML format.
# ARG: fichier  (I): File name
#      nommap   (I): Map name
#      ficimg   (I): Image file name
#      imgmap   (I): Image map : array (0..n) of lists. Each list contains
#                    5 elements : shape, coordinates, URL, ALT, TARGET
#
# RET: 1 = OK, 0= NOK, 2= nothing to update, inserted at the end
#
#========================================================================
proc TkIMhtml:UpdateImgMap { fichier nommap ficimg im } {
upvar $im imgmap

    set extension [AnkiConf::GetVar Gene ExtFicSauv "___"]

    set fichier_in [file rootname $fichier].$extension
    catch { file delete $fichier_in }
    file rename $fichier $fichier_in

    set fd_in [open $fichier_in "r"]
    set fd_out [open $fichier "w"]
    set flag_found 0
    set flag_map 0


    # Copy one file into the other, with special treatment for IMAGEMAP
    # and IMG tags

    while {[gets $fd_in ligne] >= 0} {
       set upligne [string toupper $ligne] 		;# conversion to uppercase


      # Overwrite the current MAP definition (should its
      # name match the one we're creating)

      set inddeb [string first "<MAP " $upligne]
      if { $inddeb >= 0 } {

         # Write the beginning of the line to the output file

         if { $inddeb > 0 } { puts -nonewline $fd_out [string range $ligne 0 [expr $inddeb-1]] }


         # Read the whole tag

         TkIMhtml:parseTag $fd_in ligne "MAP" tablo
         set upligne [string toupper $ligne] 		;# ligne has been modified


         # Do the names match ? If so, insert new imagemap and don't write back the old one

         if { [info exists tablo(NAME)] } {
            if { [string compare [string trim $tablo(NAME) {\"}] $nommap] == 0} {
               TkIMhtml:InsertImgMapDef $fd_out $nommap $im
               set flag_map 1
               set flag_found 1
            }
         }

         # else rewrite the tag, which has been swallowed by parseTag

         if { $flag_map != 1 } {
            TkIMhtml:rebuildTag $fd_out "MAP" tablo
         }
      }



      # Detection of the IMG tag

      set inddeb [string first "<IMG " $upligne]
      if { $inddeb >= 0 } {
         
         # Write the beginning of the line to the output file

         if { $inddeb > 0 } { puts -nonewline $fd_out [string range $ligne 0 [expr $inddeb-1]] }
         # Read the whole tag

         TkIMhtml:parseTag $fd_in ligne "IMG" tablo
         set upligne [string toupper $ligne] 		;# ligne has been modified

         # Do the names match ? If so, replace old definition by new

         if { [info exists tablo(USEMAP)] } {
            if { [string compare [string trim $tablo(USEMAP) {\"#}] $nommap] == 0} {
               set tablo(SRC)  \"$ficimg\"
            }
         }
         TkIMhtml:rebuildTag $fd_out "IMG" tablo
      }



      # Detection of the MAP definition end

      set inddeb [string first "</MAP>" $upligne]
      if { $inddeb >= 0 } {

         # discard </MAP> if the map was the one we were
         # looking for, because it has already been
         # re-inserted.

         if { $flag_map == 1 } { 
            set ligne [string range $ligne [expr $inddeb+6] end]
         }
         set flag_map 0
      }



      # In case no map has been found when we reach the end,
      # append it to the HTML source

      set inddeb [string first "</BODY>" $upligne]
      if { $inddeb >= 0 } {
         if { $flag_found == 0 } {
            puts $fd_out [string range $ligne 0 [expr $inddeb-1] ]
            TkIMhtml:InsertImgMapDef $fd_out $nommap $im
            TkIMhtml:InsertImgMapRef $fd_out $nommap $ficimg
            set ligne [string range $ligne $inddeb end]
         }
      }


      # Copy the line into the output file
      # Ignore lines between <MAP> and </MAP>

      if { $flag_map == 0} { puts $fd_out $ligne }

   }


    close $fd_in
    close $fd_out


    if { $flag_found == 0 } { 
       return 2
    } else {
       return 1
    }
}




#========================================================================
# FCT: TkIMhtml:WriteImgMap
# ROL: Writes the current imagemap definition in a file. The output is
#      in HTML format.
# ARG: fichier  (I): File name
#      nommap   (I): Map name
#      ficimg   (I): Image file name
#      imgmap   (I): Image map : array (0..n) of lists. Each list contains
#                    5 elements : shape, coordinates, URL, ALT, TARGET
# RET: none 
#
#========================================================================
proc TkIMhtml:WriteImgMap { fichier nommap ficimg im } {
upvar $im imgmap

    set fd [open $fichier "w"]

    TkIMhtml:InsertImgMapDef $fd $nommap $im 
    TkIMhtml:InsertImgMapRef $fd $nommap $ficimg 

    # Transfer rest of file

    close $fd
}








