#!/bin/sh
# Automagically invokes wish\
exec wish $0

#========================================================================
# NAM: TkImgMap
# ROL: Allows to define "imagemaps" to be used in a Web page.
#      Written in pure Tcl/Tk, TkImgMap requires Tk 8.0 or more.
#      Packages used :
#       AnkiConf (required)
#       msgcat   (required)
#       Img      (optional)
#       Balloon  (optional)
#
#                            -oOo-
#
#      This source can be distributed without any permission.
#      If you use it, tell me about it !
#
#      Ce source est distribuable sans autorisation.
#      S'il vous a servi, dites le moi !
#
# AUT: AnkiF (ankif@mailoman.com)
#
# $Id: TkImgMap.tcl,v 2.3.1.4 2000/09/17 19:57:42 fran Exp fran $
#
#========================================================================
# Before anything, locates where is TkImgMap, possibly following
# UNIX links. The found directory will be used as a prefix for
# any subsequent reference to TkImgMap private files (icons, text files,
# children Tcl scripts)

   if [ info exists _freewrap_patchLevel ] {
   # freewrapped application
	set tkimgmapDir "."
   } else {
   # normal Tcl script
   	set calledname [info script]
   	set realname $calledname
   	if { [file type $calledname] == "link" } { set realname [file readlink $calledname ] }

   	set tkimgmapDir [file dirname $realname]

   	unset calledname realname
   }



# reads the other pieces of the software

   source [file join $tkimgmapDir TkIMhtml.tcl]


# Appends TkImgMap directory in the package search path

  if [ info exists _freewrap_patchLevel ] {
	if { $tcl_platform(platform) == "windows" } {
	   lappend auto_path c:/progra~1/tcl/lib/tcl8.0/msgcat1.0 c:/progra~1/tcl/lib/tk8.0/Balloon1.0 ./AnkiConf0.3
	}
	if { $tcl_platform(platform) == "unix" } {
	   lappend auto_path /usr/lib/tcl8.0/msgcat1.0 /usr/lib/tk8.0/Balloon1.0 ./AnkiConf0.3
	}
   } else {
	lappend auto_path $tkimgmapDir
   }



# These packages are mandatory

   package require msgcat
   package require AnkiConf 0.3


# Try to load the "Img" package (allows to read JPG files). If not
# found, TkImgMap will only be able to read GIF files.

   set g_pkgLoaded(Img) [catch { package require Img }]


# The Balloon package is optional too

   set g_pkgLoaded(Balloon) [catch { package require Balloon }]
   if { $g_pkgLoaded(Balloon) != 0 } {
       proc set_balloon {w t} {}
       proc kill_balloon {w} {}
   } else {
#      namespace import ::Balloon::*
#      namespace import returns in error with Tk 8.0p2 because
#      package require doesn't "load" the namespace definition.
#      Never mind, this is less elegant but it works

       proc set_balloon {w t} { ::Balloon::set_balloon $w $t }
       proc kill_balloon {w} { ::Balloon::kill_balloon $w  }
   }
 



#---------------------------------------------------
# Global variables
#---------------------------------------------------

# Attributes of the map areas : URL, target frame, 
# alternate text. This array is indexed by the
# canvas items id's.

   set g_lien(MODIF) 0		;# This one is special.It indicates if some
                          	 # change occured since last save.
   set g_lien(URL,0) "" 	;# URL
   set g_lien(ALT,0) ""		;# ALTernate text
   set g_lien(FRAME,0) ""	;# target frame


# General attributes of the imagemap

   set g_map(NOMFIC) ""		;# name of image file (default= original image filename)
   set g_map(NOMMAP) ""		;# name of image map (default= original image filename)


# Global variables used during the construction of
# the current area.

   set debX   0			;# starting coordinates
   set debY   0
   set finX   0			;# ending coordinates
   set finY   0
   set g_poly ""


#  I need to have these variables in the global scope
#  because they're used by widgets callbacks

set g_iconList {} ;     # list of icons whose state must be updated
                        # when an image is loaded/quit
set g_dialogStatus {} ; # "ok" or "cancel" (button pressed in a
                        # non modal dialog - tk_wait always returns "")
                        # This variable is used exclusively internally
                        # by the proc which displays the dialog.


#  Yet to be sorted out

set g_htmlFile ""	;# Where to write (back) the HTML image map
set g_handleimage 0
set g_id 0
set g_forme "Rect"	;#  Current shape : "Rect", "Oval", "Poly"
set g_appName "TkImgMap"


#***********************************************************************
#                          MISCELLANEOUS FUNCTIONS                     *
#***********************************************************************
#========================================================================
# FCT: HandlerFin
# ROL: Is called before the end of the application.
# ARG: none
#
#========================================================================
proc HandlerFin {} {
global g_lien

   # Ask for confirmation if changes have been made
   set confirm ok
   if {$g_lien(MODIF)==1} {
       set confirm [ tk_messageBox -default cancel \
                     -icon question -type okcancel \
                     -message [::msgcat::mc warning:quit]]
   }

   # Save configuration file before quitting
   if {"$confirm"=="ok"} {
      catch { ::AnkiConf::WriteFile TkImgMap.ini }
      exit
   }
}



#***********************************************************************
#                     FILE MANAGEMENT FONCTIONS                        *
#***********************************************************************

#========================================================================
# FCT: LectureImage
# ROL: Reads an image from a file into a canvas. The image is aligned
#      on the upper-left side.
#
# ARG: c          (I): Canvas window path
#      fichier    (I): Name of image file to be read
#
#========================================================================
proc LectureImage { c fichier } {
global g_handleimage

catch { $c delete $g_handleimage }

# Read the image in memory
set result [image create photo monimage -file $fichier]

# Transfer the image into the canvas
set g_handleimage [$c create image 0 0 -image monimage -anchor nw]

# Scrollbars update
$c config -scrollregion "0 0 [image width monimage] [image height monimage]"

}




#========================================================================
# FCT: InitImgMap
# ROL: Reads an imagemap definition from an HTML file. 
#
# ARG: ficimg     (I): Name of image file associated to the imagemap ;
#                      passed to this function because it's only here
#                      that we're sure that the user won't cancel the
#                      load. Then this function stores it in a global
#                      variable for future use.
#      nommap     (I): name of the imagemap (see above the justification
#                      of passing it to this function)
#      c          (I): Canvas where to draw the shapes forming the imagemap
#      im         (I): Image map : array (0..n) of lists. Each list contains
#                      5 elements : shape, coordinates, URL, ALT, TARGET
#
#========================================================================
proc InitImgMap { ficimg nommap c {im {none}} } {
global g_map
global g_lien

    #---------------------------------------------------
    # Ask for confirmation if changes have been made
    #---------------------------------------------------
    set confirm ok
    if {$g_lien(MODIF)==1} {
        set confirm [ tk_messageBox -default cancel \
                      -icon question -type okcancel \
                      -message [::msgcat::mc warning:change]]
    }
    # Cancelled by user...
    if {"$confirm"!="ok"} { return 0}



    set g_map(NOMFIC) $ficimg
    set g_map(NOMMAP) $nommap


 # Reinitialize global variables

    catch { destroy g_lien }
    set g_lien(URL,0) ""
    set g_lien(ALT,0) ""
    set g_lien(FRAME,0) ""


 # Destroy canvas content (all areas have the "figure"
 # tag). The image itself is not destroyed that way, it is
 # deleted during the read of the new one.

    $c delete figure


 # If an imagemap is given, initialize canvas content

    if { "$im" != "none" } {
       upvar $im maplst
       foreach i [array names maplst] {
          RecreeForme  $c [lindex $maplst($i) 0] \
                          [lindex $maplst($i) 1] \
                          [lindex $maplst($i) 2] \
                          [lindex $maplst($i) 3] \
                          [lindex $maplst($i) 4]
       }
    }

}





#========================================================================
# FCT: LecFicImage
# ROL: .
# ARG: fichier  (I): Image file
#      c        (I): Canvas where the image will be loaded
# RET: 1 if OK, 0 if KO.
#
#========================================================================
proc LecFicImage {fichier c} {
global g_lien
global g_htmlFile



    set result [InitImgMap "$fichier" "[file root [file tail $fichier]]" $c]
    if { $result == 0} {return 0}


    # Load new image into the canvas

    LectureImage $c $fichier


    set g_lien(MODIF) 0
    set g_htmlFile ""

    return 1
}



#========================================================================
# FCT: SelecFicImage
# ROL: File selection dialog of a bitmap file (GIF or possibly JPG).
#      The bitmap is drawn in a canvas, all previously drawn areas
#      are destroyed.
# ARG: w        (I): Parent window of the dialog window
#      c        (I): Canvas where the image will be loaded
# RET: 1 if OK, 0 if KO.
#
#========================================================================
proc SelecFicImage {w c} {
global g_pkgLoaded

    #---------------------------------------------------
    # Build list of available file types
    #---------------------------------------------------
    set types ""
    lappend types [list [::msgcat::mc fic:gif]		{.gif}		]
    lappend types [list [::msgcat::mc fic:gif]		""		{GIFF}]

    # If package "Img" available, TkImgMap can load JPEG and PNG files !
    if { $g_pkgLoaded(Img) == 0 } {
      lappend types [list [::msgcat::mc fic:jpg]		{.jpg .jpeg}	]
      lappend types [list [::msgcat::mc fic:jpg]		""		{JPEG}]
      lappend types [list [::msgcat::mc fic:png]		{.png}	]
     }

    # End of list : the "all files" option is appended
    lappend types [list [::msgcat::mc fic:all]		*	]


    #---------------------------------------------------
    # Display dialog and let user choose a file
    #---------------------------------------------------
    set lastdir [::AnkiConf::GetVar Dir LastImg]

    set fichier [tk_getOpenFile -filetypes $types -parent $w -initialdir $lastdir]
    if [string compare $fichier ""] {

       set result [LecFicImage $fichier $c]

       if { $result != 0 } {
          # Save current directory with configuration package

          ::AnkiConf::CreateString Dir LastImg [file dirname $fichier]
          return 1
       }
    }
return 0
}



#========================================================================
# FCT: EcrFicHtml
# ROL: File selection dialog of an HTML file where the HTML code of
#      the imagemap will be saved (using function TkIMhtml::UpdateImgMap).
# ARG: w        (I): Parent window of the dialog window
#      c        (I): Canvas window
#      mode     (I): if = "new", the current filename (g_map(file))
#                    is ignored 
# RET: none
#
#========================================================================
proc EcrFicHtml {w c mode} {
global g_htmlFile
global g_lien
global g_map

    set saveAs  "false"

    if { ("$g_htmlFile" == "") || ("$mode" == "new") } {
      set saveAs  "true"

    # Display a dialog to confirm/modify map & file names
      set retour [EditIdMap]
      if {"$retour" != "ok"} return ;		# Cancelled
    }


 # Test overlapping areas

    if { [::AnkiConf::GetVar Config Overlap] == "yes" } {
       set retour [TestRecouv $c]
       if {"$retour" != 0} return ;		# Overlapping areas
    }

 # Ask for filename when necessary
    if { "$saveAs" == "true" } {
       set types ""
       lappend types [list [::msgcat::mc fic:html]		{.html .htm}	]
       lappend types [list [::msgcat::mc fic:html]		""		{HTML}]
       lappend types [list [::msgcat::mc fic:all]		*	]

       set lastdir [::AnkiConf::GetVar Dir LastSauv]
       set fichier [tk_getSaveFile -filetypes $types -parent $w \
                    -initialdir $lastdir -defaultextension .html]
    } else {
       set fichier $g_htmlFile
    }

    if {[string compare $fichier ""] == 0} return ;



 #  Build image map list : items of the canvas
 #  (the areas can be identified by their "figure" tag)
 #  The iamage map is built as an array of lists ; each
 #  list describes an area : type, (sub)list of coordinates,
 #  URL, ALTernate text, target FRAME

    foreach { monitem } [$c find withtag figure] {
       set imgmap($monitem) [ list [$c type $monitem] \
                                   [$c coords $monitem] \
                                   $g_lien(URL,$monitem) \
                                   $g_lien(ALT,$monitem) \
                                   $g_lien(FRAME,$monitem)]
    }



 # Write to file. When possible, the image file path is relative
 # to the HTML file.

    set g_htmlFile $fichier
    set path [file dirname $fichier]
    set longpath [string length $path]

    set ficimg $g_map(NOMFIC)

    if {[string compare [string range $g_map(NOMFIC) 0 [expr $longpath-1]] \
                        $path] == 0 } {
       set ficimg [string range $g_map(NOMFIC) [expr $longpath+1] end]
    }


    if { "$saveAs" == "true" } {
       TkIMhtml:WriteImgMap $fichier $g_map(NOMMAP) $ficimg imgmap
    } else {
       set result [TkIMhtml:UpdateImgMap $fichier $g_map(NOMMAP) $ficimg imgmap]
       if { $result == 2 } {
           tk_messageBox -icon info -type ok -message [::msgcat::mc warning:mapinsertedatend]
       }
    }

    set g_lien(MODIF) 0

    ::AnkiConf::CreateString Dir LastSauv [file dirname $fichier]
  
}



#========================================================================
# FCT: LitFicHtml
# ROL: File selection dialog allowing to import an imagemap definition
#      from a HTML file. The actual read is performed by TkIMhtml:readImgMap.
#
# ARG: w       (I): Parent window for the dialog window
#      c       (I): Canvas path
#
# RET: 1 = OK, 0 = NOK
#
#========================================================================
proc LitFicHtml {w c} {
global g_htmlFile
global g_map



# Filename selection

    set types ""
    lappend types [list [::msgcat::mc fic:html]		{.html .htm}	]
    lappend types [list [::msgcat::mc fic:html]		""		{HTML}]
    lappend types [list [::msgcat::mc fic:all]		*	]

    set lastdir [::AnkiConf::GetVar Dir LastLecHtml]
    set fichtml [tk_getOpenFile -filetypes $types -parent $w -initialdir $lastdir]

    if {[string length $fichtml] == 0} { return 0 };


 # Look for an imagemap in the file : stop on the first one.

    set result [ TkIMhtml:readImgMap $fichtml nommap maplst ]

    if { "$result" != "ok" } {
        set confirm [ tk_messageBox -icon info -type ok \
                      -message [::msgcat::mc warning:nomapinfile]]
        return 0;
    }
    set g_map(NOMMAP) $nommap


 # If the imagemap is used in the HTML file, get the
 # image file which is associated with it.

    TkIMhtml:findImgFile $fichtml $nommap ficimg


    if { [string length $g_map(NOMFIC)] > 0} {
    
       # First case : an image file has already been selected
       # via "New imagemap".
       # If the HTML file shows that the imagemap was used
       # for an image file and that image file is different
       # from the current one, ask if it should load it.
 
       if { [string length $ficimg] > 0} {
          set ficimglong [file join [file dirname $fichtml] $ficimg]
          if { [string compare $ficimglong $g_map(NOMFIC)] != 0 } {
             if [file exists $ficimglong] {
                set mess [format "[::msgcat::mc warning:noimportotherfile]" $ficimg]
                set result [tk_messageBox -icon question -type yesnocancel \
                              -message $mess]
                if { "$result" == "cancel" } { return 0 }
                if { "$result" == "yes" } {
                   set result [LecFicImage $ficimglong $c]
                   if { $result == 0 } { return 0 }
                }

             } \
             else {
                set mess [format "[::msgcat::mc warning:noimportunknownfile]" $ficimg]
                 tk_messageBox -icon info -type ok -message $mess
             }
          }
       }
    } \
    else {

       # Second case : no image file was already loaded.
       # Try to find it relatively to the HTML file directory.

       set ficimg [file join [file dirname $fichtml] $ficimg]
       if { [file exists $ficimg] } {
          LecFicImage $ficimg $c
       } \
       else {
          set mess [format "[::msgcat::mc error:importunknownfile]" $ficimg]
          tk_messageBox -icon error -type ok -message $mess
          return 0
       }
    }


 # Initializes new imagemap

    set result [InitImgMap $ficimg $nommap $c maplst]
    if { $result == 0 } { return 0 }


 # Memorize new defaults : HTML files directory (for future calls)

    set g_htmlFile $fichtml

    ::AnkiConf::CreateString Dir LastLecHtml [file dirname $fichtml]

    return 1
}



#***********************************************************************
#              USER INTERFACE MANAGEMENT (EXCLUDING CANVAS)            *
#***********************************************************************

#========================================================================
# FCT: MajIcones
# ROL: Update icons states.
# ARG: icn_list  (I): List of widgets to update
#      etat      (I): New state (normal, disabled)
# RET: none
#
#========================================================================
proc MajIcones { listName etat} {
upvar $listName iconList

foreach { nomicon } $iconList { $nomicon configure -state $etat }
}



#========================================================================
# FCT: MajTitre
# ROL: Update window title. This function is called whenever an element
#      of the title is changed (through "trace variable").
#
# ARG: icn_list  (I): List of widgets to update
#      etat      (I): New state (normal, disabled)
# RET: none
#
#========================================================================
proc MajTitre { var1 var2 op} {
global g_appName
global g_htmlFile
global g_map

   if { $g_map(NOMFIC) != "" } {
      if { $g_htmlFile != "" } {
         wm title . "$g_appName - [file tail $g_htmlFile] ([file tail $g_map(NOMFIC)])"
      } else {
         wm title . "$g_appName - [::msgcat::mc newimg] ([file tail $g_map(NOMFIC)])"
      }
   } else {
      wm title . "$g_appName"
   }
}


#========================================================================
# FCT: EditUrl
# ROL: Display a dialog to define an area's attributes.
# ARG: id      (I): Area id (array index in g_lien)
# RET: none
#
#========================================================================
proc EditUrl { id } {
global g_lien


    if {$id == ""} { set id 0 }
    toplevel .tl
    wm title .tl [::msgcat::mc editurl:title]

    label .tl.laburl -text [::msgcat::mc editurl:url]
    entry .tl.url -width 40
    .tl.url insert end "$g_lien(URL,$id)"

    label .tl.labalt -text [::msgcat::mc editurl:alt]
    entry .tl.alt -width 40
    .tl.alt insert end "$g_lien(ALT,$id)"

    label .tl.labtarg -text [::msgcat::mc editurl:frame]
    entry .tl.targ -width 40
    .tl.targ insert end "$g_lien(FRAME,$id)"

    frame .tl.box
    set cmd [concat set g_lien(URL,$id) {[.tl.url get]} ";" \
             set g_lien(ALT,$id) {[.tl.alt get]} ";" \
             set g_lien(FRAME,$id) {[.tl.targ get]} ";" \
             grab release .tl ";" destroy .tl]
    button .tl.box.ok -text [::msgcat::mc OK] -command "$cmd" \
                      -width 6 -default active
    button .tl.box.cancel -text [::msgcat::mc Cancel] -command {grab release .tl ; destroy .tl} -width 6
    pack .tl.box.ok .tl.box.cancel -padx 10 -side left
    pack .tl.laburl .tl.url .tl.labalt .tl.alt .tl.labtarg .tl.targ -anchor w -side top -padx 10
    pack .tl.box -side top -pady 10

    bind .tl <Return> ".tl.box.ok invoke"
    bind .tl <Escape> ".tl.box.cancel invoke"

    tkwait visibility .tl
    focus .tl.url
    grab .tl
}


#========================================================================
# FCT: EditIdMap
# ROL: Display a dialog window to confirm/change the map & file names.
# ARG: none
# RET: "ok" if OK, "cancel" if dialog cancelled by user
#
#========================================================================
proc EditIdMap { } {
global g_map
global g_dialogStatus

    toplevel .tl
    wm title .tl [::msgcat::mc editmap:title]


    # -- image file name and imagemap name can be edited

    label .tl.labfile -text [::msgcat::mc editmap:file]
    entry .tl.file -width 40
    .tl.file insert end "$g_map(NOMFIC)"

    label .tl.labmap -text [::msgcat::mc editmap:name]
    entry .tl.map -width 40
    .tl.map insert end "$g_map(NOMMAP)"


    # -- OK and Cancel buttons

    frame .tl.box
    set cmd [concat set g_map(NOMFIC) {[.tl.file get]} ";" \
             set g_map(NOMMAP) {[.tl.map get]} ";" set g_dialogStatus ok ";" \
             grab release .tl ";" destroy .tl]
    button .tl.box.ok -text [::msgcat::mc OK] -width 6 -default active \
                      -command "$cmd"

    button .tl.box.cancel -text [::msgcat::mc Cancel] -width 6 \
                      -command "set g_dialogStatus cancel ; grab release .tl ; destroy .tl"

    pack .tl.box.ok .tl.box.cancel -padx 10 -side left
    pack .tl.labfile .tl.file .tl.labmap .tl.map -anchor w -side top -padx 10
    pack .tl.box -side top -pady 10

    bind .tl <Return> ".tl.box.ok invoke"
    bind .tl <Escape> ".tl.box.cancel invoke"

    focus .tl.file
    grab .tl
    tkwait window .tl

    return $g_dialogStatus
}



#========================================================================
# FCT: APropos
# ROL: Display a small info window
# ARG: none
# RET: none
#
#========================================================================
proc APropos {} {
    tk_messageBox -icon info -type ok -title "[::msgcat::mc about:title]"\
                  -message "[::msgcat::mc about:text]\n\n\
                            AnkiF : ankif@mailoman.com"
}




#***********************************************************************
#                     CANVAS MANAGEMENT FUNCTIONS                      *
#***********************************************************************

#========================================================================
# FCT: OuSuisJe
# ROL: Where am I ? (finds the id of the area under the mouse cursor)
# ARG: c        (I): Canvas
# RET: Area id, or 0 if no area is under the mouse cursor
#
#========================================================================
proc OuSuisJe { c } {
global g_handleimage

    set numitem [$c find withtag current]
    if {$numitem == $g_handleimage} {
       set numitem 0
      }
    return $numitem
}


#========================================================================
# FCT: MajCanvas
# ROL: Enable/disable user actions in the canvas
# ARG: c         (I): Canvas
#      etat      (I): New state (normal, disabled)
# RET: none
#
#========================================================================
proc MajCanvas {c etat} {

if { $etat == "normal" } {
   bindtags $c "$c Canvas . all"
  } \
else {
   bindtags $c "Canvas . all"
  }
}




#========================================================================
# FCT: SelecCouleur
# ROL: Color selection dialog window. The color will be used for subsequent
#      drawings.
# ARG: none
# RET: new color
#
#========================================================================
proc SelecCouleur {  } {

    set couleur [::AnkiConf::GetVar Config CouleurForme]
    set nouvelle [tk_chooseColor -initialcolor $couleur]
    if { $nouvelle != "" } {
       set couleur $nouvelle
       ::AnkiConf::SetVar Config CouleurForme $couleur
    }

    return $couleur
}



#========================================================================
# FCT: DebutForme
# ROL: Left mouse click : first corner of a new area. For polys,
#     other edges may have been drawn already.
# ARG: c        (I): Canvas
#      x, y     (I): Mouse coordinates
# RET: none
#
#========================================================================
proc DebutForme {c x y} {
global debX debY finX finY
global g_poly
global g_forme

    # traitement particulier polygones
    if { [llength $g_poly] > 0 } return

    set debX [$c canvasx $x]
    set debY [$c canvasy $y]
    set finX $debX
    set finY $debY
    $c delete elastique

    if { $g_forme == "Poly" } {
       set g_poly [list $debX $debY]
      }

}



#========================================================================
# FCT: CreeRect
# ROL: Procedure called during the area creation (while Button-1 is
#      held down and mouse is moved): draws a rectangle tracking the
#      mouse movement.
# ARG: c        (I): Canvas
#      x, y     (I): Mouse coordinates
# RET: none
#
#========================================================================
proc CreeRect {c x y} {
global debX debY finX finY

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    if {($debX != $x) && ($debY != $y)} {
	$c delete elastique
	$c addtag elastique withtag [$c create rect $debX $debY $x $y \
		-outline white]
	set finX $x
	set finY $y
    }
}

#========================================================================
# FCT: CreeCercle
# ROL: Procedure called during the area creation (while Button-1 is
#      held down and mouse is moved): draws a circle tracking the
#      mouse movement.
# ARG: c        (I): Canvas
#      x, y     (I): Mouse coordinates
# RET: none
#
#========================================================================
proc CreeCercle {c x y} {
global debX debY finX finY

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    if {($debX != $x) || ($debY != $y)} {
	$c delete elastique
      set xdelta [expr abs($x - $debX)]
      set ydelta [expr abs($y - $debY)]
      if { $x > $debX } { set xsens 1 } else { set xsens -1 }
      if { $y > $debY } { set ysens 1 } else { set ysens -1 }
      if { $xdelta > $ydelta } {
         set finX $x
         set finY [expr ($debY + ($ysens * $xdelta))] 
        } \
      else {
         set finY $y
         set finX [expr ($debX + ($xsens * $ydelta))] 
        }
	$c addtag elastique withtag [$c create oval $debX $debY $finX $finY \
		-outline white]
    }
}

#========================================================================
# FCT: CreePoly
# ROL: Procedure called during the area creation (while Button-1 is
#      held down and mouse is moved): draws one edge of the polygon,
#      by tracking the mouse movement.
# ARG: c        (I): Canvas
#      x, y     (I): Mouse coordinates
# RET: none
#
#========================================================================
proc CreePoly {c x y} {
global debX debY finX finY

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    if {($debX != $x) || ($debY != $y)} {
      set finX $x
      set finY $y
      $c delete elastique
	$c create line $debX $debY $finX $finY -fill white -tags elastique
    }
}



#========================================================================
# FCT: FinRect
# ROL: Button-1 is released : end of rectangular area creation.
# ARG: c        (I): Canvas
# RET: none
#
#========================================================================
proc FinRect {c} {
global debX debY finX finY
global g_lien

    if {($debX != $finX) && ($debY != $finY)} {
	 $c delete elastique
       set couleur [::AnkiConf::GetVar Config CouleurForme]
       set id [$c create rect $debX $debY $finX $finY \
               -outline $couleur -fill gray -stipple gray12 -tags figure ]
       set g_lien(URL,$id) "http://"
       set g_lien(ALT,$id) ""
       set g_lien(FRAME,$id) ""
       set g_lien(MODIF) 1
      }
}



#========================================================================
# FCT: FinCercle
# ROL: Button-1 is released : end of circular area creation.
# ARG: c        (I): Canvas
# RET: none
#
#========================================================================
proc FinCercle {c} {
global debX debY finX finY
global g_lien

    if {$debX != $finX} {
	$c delete elastique
       set couleur [::AnkiConf::GetVar Config CouleurForme]
       set id [$c create oval $debX $debY $finX $finY \
               -outline $couleur -fill gray -stipple gray12 -tags figure ]
       set g_lien(URL,$id) "http://"
       set g_lien(ALT,$id) ""
       set g_lien(FRAME,$id) ""
       set g_lien(MODIF) 1
      }
}

#========================================================================
# FCT: FinPoly
# ROL: Button-1 is released : end of polygon edge creation. If current
#      point is "near" the first point, the polygon is closed.
# ARG: c        (I): Canvas
# RET: none
#
#========================================================================
proc FinPoly {c} {
global debX debY finX finY
global g_lien
global g_poly

    if {($debX != $finX) || ($debY != $finY)} {
	$c delete elastique
      set x0 [lindex $g_poly 0]
      set y0 [lindex $g_poly 1]
      set seuil [::AnkiConf::GetVar Config DeltaPoly]

      if { ([expr abs($finX-$x0)]<$seuil) && ([expr abs($finY-$y0)]<$seuil) } {
          $c delete poly
          set couleur [::AnkiConf::GetVar Config CouleurForme]
          set id [eval "$c create poly $g_poly \
               -outline $couleur -fill gray -stipple gray12 -tags figure" ]
          set g_poly ""

          set g_lien(URL,$id) "http://"
          set g_lien(ALT,$id) ""
          set g_lien(FRAME,$id) ""
          set g_lien(MODIF) 1
         } \
      else {      
          $c create line $debX $debY $finX $finY -tags poly
          lappend g_poly $finX $finY
          set debX $finX
          set debY $finY
         }
      }
}


#========================================================================
# FCT: RecreeForme
# ROL: Re-create an area given its type, coordinates and attributes.
# ARG: c         (I): Canvas
#      shape     (I): Area shape ("rect", "circle", "poly")
#      coords    (I): List of coordinates : 2 points for a rectangle
#                     (4 numbers), 1 point and a radius for a circle
#                     (3 numbers), x points for a polygon (2 * x numbers,
#                     x >= 3)
#      url       (I): Attribute : URL
#      alt       (I): Attribute : alternate text
#      target    (I): Attribute : target frame
#
#========================================================================
proc RecreeForme { c shape coords url alt target } {
global g_lien

    set id 0
    set couleur [::AnkiConf::GetVar Config CouleurForme]
    switch $shape {
       rect    { 
                 if {[llength $coords] == 4 } {
                    set id [eval "$c create rect $coords \
                    -outline $couleur -fill gray -stipple gray12 -tags figure" ]
                   }
               } 
       circle  { 
                 if {[llength $coords] == 3 } {
                    set debX  [expr ([lindex $coords 0]-[lindex $coords 2])]
                    set finX  [expr ([lindex $coords 0]+[lindex $coords 2])]
                    set debY  [expr ([lindex $coords 1]-[lindex $coords 2])]
                    set finY  [expr ([lindex $coords 1]+[lindex $coords 2])]

                    set id [$c create oval $debX $debY $finX $finY \
                   -outline $couleur -fill gray -stipple gray12 -tags figure ]
                   }
               } 
      default {
               if {[llength $coords] >= 6 } {
                  set id [eval "$c create poly $coords \
                     -outline $couleur -fill gray -stipple gray12 -tags figure" ]
                   }
               } 
         }
      set g_lien(URL,$id) $url
      set g_lien(ALT,$id) $alt
      set g_lien(FRAME,$id) $target
}



#========================================================================
# FCT: DebutBouge
# ROL: Left mouse click in a figure. Initialise movement.
# ARG: c        (I): Canvas
#      x, y     (I): Mouse coordinates
# RET: none
#
#========================================================================
proc DebutBouge {c x y} {
global debX debY finX finY

    set debX [$c canvasx $x]
    set debY [$c canvasy $y]
    set finX $debX
    set finY $debY

    $c bind all <Any-Enter> "break"
    $c bind all <Any-Leave> "break"
    MajCanvas $c disabled
}


#========================================================================
# FCT: BougeItem
# ROL: Move an already defined area. The area is the "current" one
#      (according to the canvas object).
# ARG: c       (I): Canvas
#      x, y    (I): New coordinates.
#
#========================================================================
proc BougeItem {c x y} {
global debX debY
global g_lien

    set x [$c canvasx $x]
    set y [$c canvasy $y]
    $c move current [expr $x-$debX] [expr $y-$debY]
    set debX $x
    set debY $y
    set g_lien(MODIF) 1
}



#========================================================================
# FCT: FinBouge
# ROL: Left mouse released
# ARG: c        (I): Canvas
# RET: none
#
#========================================================================
proc FinBouge {c} {

    $c bind all <Any-Enter> ""
    $c bind all <Any-Leave> ""
    after idle "MajCanvas $c normal"
}

#========================================================================
# FCT: DetruitItem
# ROL: Destroy an area and associated attributes.
# ARG: c       (I): Canvas
#      id      (I): Area id (item id for the canvas)
#
#========================================================================
proc DetruitItem {c id} {
global g_lien
    
    $c delete $id
    unset g_lien(URL,$id)
    unset g_lien(ALT,$id)
    set g_lien(MODIF) 1
}



#========================================================================
# FCT: EntreeItem
# ROL: Updates the status bar when the mouse is over a new area in the
#      canvas.
# ARG: c        (I): Canvas
#      statbar  (I): Status bar window path
# RET: none
#
#========================================================================
proc EntreeItem { c statbar } {
    global g_lien

    set id [OuSuisJe $c]
    if { $id != "" } {
       $statbar.url config -text "URL: $g_lien(URL,$id)"
       $statbar.alt config -text "$g_lien(ALT,$id)"
      }

   # Set a new mouse cursor
   $c config -cursor fleur
}


#========================================================================
# FCT: SortieItem
# ROL: Updates the status bar when the mouse has left the current area
#      in the canvas.
# ARG: c        (I): Canvas
#      statbar  (I): Status bar window path
# RET: none
#
#========================================================================
proc SortieItem { c statbar } {

    $statbar.url config -text " "
    $statbar.alt config -text " "
   $c config -cursor left_ptr
}



#========================================================================
# FCT: TesteRecouv
# ROL: Look for overlapping areas.
# ARG: c        (I): Canvas
# RET: 0 if no overlapping areas, 1 otherwise 
#
#========================================================================
proc TestRecouv { c } {

    set erreur 0

    # all the areas are tagged with the "figure" tag on creation.

    foreach { monitem } [$c find withtag figure] {
       set zone [$c bbox $monitem]

       set cmd [concat $c find overlapping $zone]
       set liste [eval $cmd]

       # Only 1 intersection allowed : the item and the image
       if "[llength $liste] > 2" {
          tk_messageBox -message [::msgcat::mc error:overlap]
          set erreur 1
          break
         }
      }

   return $erreur
}




#========================================================================
# FCT: InitIhm
# ROL: Create windows and widgets
# ARG: langue    (I): user language
# RET: none
#
#========================================================================
proc InitIhm { appName langue } {
global g_id
global g_iconList
global tkimgmapDir


#---------------------------------------------------
# Define 3 windows :
# - top     = icon bar
# - middle  = canvas
# - bottom  = status bar
#---------------------------------------------------
set pico [frame .panel1 -relief ridge]
set fgraph [frame .panel2]
set pstat [frame .panel3]


#---------------------------------------------------
# Canvas
#---------------------------------------------------
set cgraph $fgraph.canevas
canvas $cgraph -relief sunken -borderwidth 2 \
	-xscrollcommand "$fgraph.hscroll set" \
	-yscrollcommand "$fgraph.vscroll set"
scrollbar $fgraph.hscroll -orient horiz -command "$cgraph xview"
scrollbar $fgraph.vscroll -command "$cgraph yview"

# --  The grid manager is excellent to painlessly
#     add scrollbars to the canvas

grid rowconfig    $fgraph 0 -weight 1 -minsize 0
grid columnconfig $fgraph 0 -weight 1 -minsize 0

grid $cgraph -padx 1 -in $fgraph -pady 1 \
    -row 0 -column 0 -rowspan 1 -columnspan 1 -sticky nsew
grid $fgraph.vscroll -in $fgraph -padx 1 -pady 1 \
    -row 0 -column 1 -rowspan 1 -columnspan 1 -sticky nsew
grid $fgraph.hscroll -in $fgraph -padx 1 -pady 1 \
    -row 1 -column 0 -rowspan 1 -columnspan 1 -sticky nsew

# Canvas context menu
# -------------------
# (g_id is set on right mouse click)

set menu_context [menu .popmen -tearoff 0]
$menu_context add command -label [::msgcat::mc menu:define]\
            -command {eval {EditUrl $g_id}}
# Bad construction but I don't want to set cgraph as global
set cmd [format "DetruitItem %s \$g_id ; SortieItem %s %s" \
         $cgraph $cgraph $pstat] 
$menu_context add command -label [::msgcat::mc menu:destroy] \
            -command $cmd


# Canvas callbacks
# ----------------
set cmd [format "set g_id \[OuSuisJe %s\]" $cgraph]

$cgraph bind figure <Any-Enter> "EntreeItem $cgraph $pstat"
$cgraph bind figure <Any-Leave> "SortieItem $cgraph $pstat"
$cgraph bind figure <ButtonPress-3> "$cmd; $menu_context post %X %Y"
$cgraph bind figure <Double-1> "$cmd ; $menu_context invoke 0"
$cgraph bind figure <ButtonPress-1> "DebutBouge $cgraph %x %y; break"
$cgraph bind figure <B1-Motion> "BougeItem $cgraph %x %y; break"
$cgraph bind figure <ButtonRelease-1> "FinBouge $cgraph; break"

bind $cgraph <ButtonPress-1> "DebutForme $cgraph %x %y"
bind $cgraph <B1-Motion> [join [list {Cree$g_forme} $cgraph %x %y]]
bind $cgraph <ButtonRelease-1> [join [list {Fin$g_forme} $cgraph]]

$cgraph configure -cursor left_ptr


#---------------------------------------------------
# Status bar
#---------------------------------------------------
label $pstat.alt -relief sunken -padx 3 -pady 2 -anchor w -width 25
label $pstat.url -relief sunken -padx 3 -pady 2 -anchor w
pack $pstat.alt -side left -anchor w
pack $pstat.url -side left -anchor w -expand yes -fill x


#---------------------------------------------------
# Commands to be associated with buttons
# and/or icons
#---------------------------------------------------
# "New" command : if the read is successful, updates the GUI elements
set cmd_new [format "if \[SelecFicImage . %s\] {MajIcones g_iconList normal ; MajCanvas %s normal }" $cgraph $cgraph]

# "Open" command : just reads the file
#set cmd_open "LitFicHtml . $cgraph"
set cmd_open [format "if \[LitFicHtml . %s\] {MajIcones g_iconList normal ; MajCanvas %s normal }" $cgraph $cgraph]

# "Save" and "Save As" commands
set cmd_save "EcrFicHtml . $cgraph {}"
set cmd_saveAs "EcrFicHtml . $cgraph new"

set cmd_setcolor [format "$pico.couleur config -bg \[SelecCouleur\]"]


#---------------------------------------------------
# Application menu
#---------------------------------------------------
set menubar [menu .menu]

# ---------- "File" menu ---------------
set menufic [menu $menubar.entree1 -tearoff 0]
$menufic add command -label [::msgcat::mc menu:new] -command $cmd_new
$menufic add command -label [::msgcat::mc menu:open] -command $cmd_open
$menufic add command -label [::msgcat::mc menu:save] -command $cmd_save
$menufic add command -label [::msgcat::mc menu:saveAs] -command $cmd_saveAs
$menufic add separator
$menufic add command -label [::msgcat::mc menu:quit] -command "HandlerFin" -accelerator "Alt-X"
bind . <Alt-x> HandlerFin



# ---------- "Tools" menu --------------
set menutool [menu $menubar.entree2 -tearoff 0]

# ... "Shape" sub-menu ...
set submenu [menu $menutool.casc1 -tearoff 0]
$submenu add command -label [::msgcat::mc forme:rect] \
                     -command "set g_forme Rect" 
$submenu add command -label [::msgcat::mc forme:cercle] \
                     -command "set g_forme Cercle" 
$submenu add command -label [::msgcat::mc forme:poly] \
                     -command "set g_forme Poly"

# ... "Preferences" sub-menu ...
set submenu [menu $menutool.casc2 -tearoff 0]
$submenu add command -label [::msgcat::mc menu:confgen] \
                     -command "::AnkiConf::EditDialog . Gene \
                              [::msgcat::mc menu:prefs]:[::msgcat::mc menu:confgen]"
$submenu add command -label [::msgcat::mc menu:dessin] \
                     -command "::AnkiConf::EditDialog . Config \
                              [::msgcat::mc menu:prefs]:[::msgcat::mc menu:dessin]"

# ... assembling the menu ...
$menutool add command -label [::msgcat::mc menu:couleur] -command $cmd_setcolor
$menutool add cascade -menu $menutool.casc1 -label [::msgcat::mc menu:forme]
$menutool add separator
$menutool add cascade -menu $menutool.casc2 -label [::msgcat::mc menu:prefs]


# ------------ "Help" menu ------------------
set menuhelp [menu $menubar.help -tearoff 0]
$menuhelp add command -label [::msgcat::mc menu:usage] -command "source [file join $tkimgmapDir TkIMhelp.tcl]"
$menuhelp add command -label [::msgcat::mc menu:apropos] -command APropos

# Now we assemble these menus into the main menu
$menubar add cascade -menu $menufic -label [::msgcat::mc menu:file] 
$menubar add cascade -menu $menutool -label [::msgcat::mc menu:tools]
$menubar add cascade -menu $menuhelp -label [::msgcat::mc menu:help]

. configure -menu $menubar


#---------------------------------------------------
# Icons
#---------------------------------------------------
# "New", "Open" and "Save" icons

image create photo im_new -file [file join $tkimgmapDir icons new.gif]
button $pico.new -image im_new -command $cmd_new
set_balloon $pico.new [::msgcat::mc help:open]

image create photo im_open -file [file join $tkimgmapDir icons open.gif]
button $pico.open -image im_open -command $cmd_open
set_balloon $pico.open [::msgcat::mc help:import]

image create photo im_save -file [file join $tkimgmapDir icons save.gif]
button $pico.save -image im_save -command $cmd_save
set_balloon $pico.save [::msgcat::mc help:save]
lappend g_iconList $pico.save


# "Rectangle", "Circle", "Polygon" icons

image create photo im_rect -file [file join $tkimgmapDir icons rectangle.gif]
radiobutton $pico.rect -image im_rect -indicatoron false -selectcolor "" \
            -variable g_forme -value "Rect"
set_balloon $pico.rect [::msgcat::mc help:shape]
lappend g_iconList $pico.rect

image create photo im_cerc -file [file join $tkimgmapDir icons cercle.gif]
radiobutton $pico.cerc -image im_cerc -indicatoron false -selectcolor "" \
             -variable g_forme -value "Cercle"
set_balloon $pico.cerc [::msgcat::mc help:shape]
lappend g_iconList $pico.cerc

image create photo im_poly -file [file join $tkimgmapDir icons polygone.gif]
radiobutton $pico.poly -image im_poly -indicatoron false -selectcolor "" \
             -variable g_forme -value "Poly"
set_balloon $pico.poly [::msgcat::mc help:shape]
lappend g_iconList $pico.poly


# "Color" selector icon

image create photo im_couleur -file [file join $tkimgmapDir icons couleur.gif]
button $pico.couleur -image im_couleur -command $cmd_setcolor -bg [::AnkiConf::GetVar Config CouleurForme]
set_balloon $pico.couleur [::msgcat::mc help:color]
lappend g_iconList $pico.couleur

# "Map identification" icon

image create photo im_mapident -file [file join $tkimgmapDir icons ident.gif]
button $pico.mapident -image im_mapident -command EditIdMap
set_balloon $pico.mapident [::msgcat::mc help:mapident]
lappend g_iconList $pico.mapident


pack $pico.new $pico.open -side left
pack $pico.save -side left -padx 10
pack $pico.rect $pico.cerc $pico.poly -side left -expand 0
pack $pico.couleur -side left -padx 10
pack $pico.mapident -side left -padx 10

# initial settings = rectangle with inactive menu
$pico.rect invoke
MajIcones g_iconList "disabled"
MajCanvas $cgraph "disabled"


#---------------------------------------------------
# Global layout : pack the 3 frames
#---------------------------------------------------
pack $pico -side top -expand no -fill x -pady 2
pack $fgraph -expand yes -fill both -padx 1 -pady 1
pack $pstat -side top -expand no -fill x

wm title . "$appName"
wm iconname . "$appName"
wm protocol . WM_DELETE_WINDOW HandlerFin


}



#========================================================================
# FCT: InitDefaults
# ROL: Initialize default configuration if TkImgMap.ini doesn't exists.
# ARG: none
# RET: 0 
#
#========================================================================
proc InitDefaults { } {

  ::AnkiConf::CreateString Config CouleurForme red
  ::AnkiConf::CreateVar Config Overlap "yes" "bool"
  ::AnkiConf::CreateInt Config DeltaPoly 10

#  ::AnkiConf::CreateVar Gene Langue en "list(fr,en)"
  ::AnkiConf::CreateString Gene ExtFicSauv bck  
}


#========================================================================
# FCT: -
# ROL: Main program
# ARG: none
#
#========================================================================

   # Read preferences
   set fn "TkImgMap.ini"

   if [file exists $fn] {
      ::AnkiConf::ReadFile $fn
   } else {
      InitDefaults
   }

   # Look for language configuration files in the "text" subdir
   # Initialise selection menu with what has been found

   set countryCodes ""
   set langue [::AnkiConf::GetVar Gene Langue]

   set fileList [glob [file join text *.msg]]
   foreach i $fileList { 
      append countryCodes [file rootname [file tail $i]] ","
   }

   if { $langue == "" } { set langue en }
  ::AnkiConf::CreateVar Gene Langue $langue "list($countryCodes)"



   # Use msgcat to configure TkImgMap depending on the language pref.

   ::msgcat::mclocale $langue
   ::msgcat::mcload [file join $tkimgmapDir "text"]


   # put trace on variables which appear in the window title,
   # in order to automatically update it on every change.

   trace variable g_htmlFile w MajTitre
   trace variable g_map(NOMFIC) w MajTitre


   # Start GUI

   InitIhm $g_appName $langue
